/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2012 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.json.actions;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.IOUtils;
import com.openexchange.ajax.container.ByteArrayFileHolder;
import com.openexchange.ajax.requesthandler.AJAXRequestData;
import com.openexchange.ajax.requesthandler.AJAXRequestResult;
import com.openexchange.ajax.requesthandler.DispatcherNotes;
import com.openexchange.documentconverter.Feature;
import com.openexchange.documentconverter.IManager;
import com.openexchange.documentconverter.Properties;
import com.openexchange.office.tools.FileHelper;
import com.openexchange.office.tools.Resource;
import com.openexchange.office.tools.ResourceManager;
import com.openexchange.server.ServiceLookup;
import com.openexchange.tools.session.ServerSession;

/**
 * {@link GetFileAction}
 *
 * @author <a href="mailto:firstname.lastname@open-xchange.com">Firstname Lastname</a>
 */
/*
 * MH/KA compile fix for buildsystem
 * @Action(method = RequestMethod.GET, name = "getfile", description =
 * "Get the whole document as PDF file or get the substream of an infostore file in its naive format.", parameters = {
 * @Parameter(name = "session", description = "A session ID previously obtained from the login module."),
 * @Parameter(name = "id", description = "Object ID of the requested infoitem."),
 * @Parameter(name = "folder_id", description = "Folder ID of the requested infoitem."),
 * @Parameter(name = "uid", description = "The unique id of the client application."),
 * @Parameter(name = "version", optional=true, description =
 * "If present, the infoitem data describes the given version. Otherwise the current version is returned."),
 * @Parameter(name = "filter_format", optional=true, description =
 * "If this value is set to 'pdf', the whole document is as converted PDF file, in all other cases, the content of the document's substream, described by the parameter 'fragment, is returned as unmodified file."
 * ),
 * @Parameter(name = "fragment", optional=true, description =
 * "If this value is set and the filter_format is not set or not set to 'pdf', this parameter describes the substream name of the document internal file to be returned."
 * ),
 * @Parameter(name = "filename", optional=true, description =
 * "If present, this parameter contains the name of the infostore item to be used as initial part of the filename for the returned file, other the filename is set to 'file'."
 * ), }, responseDescription = "Response with timestamp: The content of the documents substream in its native format, or converted to PNG in
 * the case of EMF, WMF and SVM graphic file formats)
 */
@DispatcherNotes(defaultFormat = "file", allowPublicSession = true)
public class GetFileAction extends DocumentFilterAction {

    /**
     * Initializes a new {@link GetFileAction}.
     *
     * @param services
     * @param oqm
     */
    public GetFileAction(ServiceLookup services, ResourceManager resMgr) {
        super(services);
        resourceManager = resMgr;
    }

    /*
     * (non-Javadoc)
     * @see com.openexchange.ajax.requesthandler.AJAXActionService#perform(com.openexchange.ajax.requesthandler.AJAXRequestData,
     * com.openexchange.tools.session.ServerSession)
     */
    @Override
    public AJAXRequestResult perform(AJAXRequestData request, ServerSession session) {
        AJAXRequestResult requestResult = null;
        String resourceName = request.getParameter("get_filename");

        if ((null != resourceName) && (resourceName.length() > 0)) {
            // sanitize resource name
            if (resourceName.startsWith("./")) {
                resourceName = resourceName.substring(2);
            }

            final String filename = FileHelper.getBaseName(resourceName);

            // try to get the graphic from the resource manager(s) first
            if (filename.startsWith("uid")) {
                // 1. try: resource is already contained within this nodes resource manager
                final long uid = Resource.getUIDFromResourceName(filename);
                final Resource nodeResource = resourceManager.getResource(uid);
                byte[] resdata = ((nodeResource != null) ? nodeResource.getBuffer() : null);

                // 2. try: create a managed resource with the given uid
                if (null == resdata) {
                    try {
                        final String managedId = Resource.getManagedIdFromUID(uid);

                        if ((null != managedId) && (managedId.length() > 0)) {
                            final Resource managedResource = Resource.createFrom(m_services, managedId);

                            // add a new managed resource to this nodes resource manager for later speedup
                            if (null != (resdata = ((null != managedResource) ? managedResource.getBuffer() : null))) {
                                resourceManager.addResource(managedResource);
                            }
                        }
                    } catch (Exception e) {
                        //
                    }
                }

                // if we've got the resource data, set it as request result
                if (null != resdata) {
                    try {
                        final String mimeType = FileHelper.getMimeType(resourceName);
                        final ByteArrayFileHolder fileHolder = new ByteArrayFileHolder(resdata);

                        fileHolder.close();
                        fileHolder.setName(resourceName);
                        fileHolder.setContentType(mimeType);
                        requestResult = new AJAXRequestResult(fileHolder, "file");
                    } catch (Exception e) {
                        //
                    }
                }
            }

            // try getting the graphic from the document's zip archive
            if (null == requestResult) {
                // always use the current version of the document
                request.putParameter("version", "0");
                InputStream documentStm = FileHelper.getDocumentStream(m_services, session, request);

                if (null != documentStm) {
                    final String[] mimeType = { "" };
                    final InputStream resultStm = getZipEntryStream(documentStm, resourceName, mimeType);

                    IOUtils.closeQuietly(documentStm);

                    if (null != resultStm) {
                        byte[] buffer = null;

                        try {
                            buffer = IOUtils.toByteArray(resultStm);
                        } catch(IOException e) {
                            //
                        } finally {
                            IOUtils.closeQuietly(resultStm);
                        }

                        if (null != buffer) {
                            requestResult = createFileRequestResult(request, buffer, mimeType[0], null);
                        }
                    }
                }
            }
        }

        return requestResult;
    }

    /**
     * @param documentStm
     * @param resourceName
     * @param mimeType
     * @param closeDocumentStream
     * @return
     */
    final private InputStream getZipEntryStream(InputStream documentStm, String resourceName, String[] mimeType) {
        InputStream resultStm = null;

        if (null != documentStm) {
            final ZipInputStream zipInputStm = new ZipInputStream(documentStm);
            final String filename = FileHelper.getBaseName(resourceName);
            String uidResourceName = null;

            // check for uid Resources within all entries
            if (filename.startsWith("uid") && (filename.length() > 3)) {
                uidResourceName = filename;
            }

            try {
                for (ZipEntry zipEntry = null; (null == resultStm) && ((zipEntry = zipInputStm.getNextEntry()) != null);) {
                    final String curEntryName = zipEntry.getName();

                    if (curEntryName.equals(FileHelper.OX_RESCUEDOCUMENT_DOCUMENTSTREAM_NAME)) {
                        // check for resources contained within the original rescue document
                        byte[] entryBuffer = null;

                        try {
                            entryBuffer = IOUtils.toByteArray(zipInputStm);
                        } catch (IOException e) {
                            //
                        }

                        if (null != entryBuffer) {
                            final InputStream entryInputStream = new ByteArrayInputStream(entryBuffer);
                            resultStm = getZipEntryStream(entryInputStream, resourceName, mimeType);
                            IOUtils.closeQuietly(entryInputStream);
                        }
                    } else if (curEntryName.equals(resourceName) || ((null != uidResourceName && curEntryName.endsWith(uidResourceName)))) {
                        // check for resources equal the the given name and for resources matching the uid resource manager naming schema
                        byte[] entryBuffer = null;

                        try {
                            entryBuffer = IOUtils.toByteArray(zipInputStm);
                        } catch (IOException e) {
                            //
                        }

                        if (null != entryBuffer) {
                            final String fileExtension = FilenameUtils.getExtension(resourceName).trim().toLowerCase();

                            // check if we create a temporary png file that can be displayed within the browser,
                            // we will do this for the formats "wmf and emf... TODO: the resulting graphic should
                            // somehow be part of the resourcemanager to prevent unnecessary graphic conversions
                            if (fileExtension.equals("emf") || fileExtension.equals("wmf") || (resourceName.indexOf("ObjectReplacements/") > -1)) {
                                final IManager dcManager = m_services.getService(IManager.class);

                                if ((null != dcManager) && dcManager.hasFeature(Feature.DOCUMENTCONVERTER)) {
                                    final HashMap<String, Object> jobProperties = new HashMap<String, Object>(8);
                                    final HashMap<String, Object> resultProperties = new HashMap<String, Object>(8);
                                    final InputStream inputStm = new ByteArrayInputStream(entryBuffer);

                                    jobProperties.put(Properties.PROP_INPUT_STREAM, inputStm);
                                    jobProperties.put(Properties.PROP_MIME_TYPE, "image/png");

                                    resultStm = dcManager.convert("graphic", jobProperties, resultProperties);
                                    IOUtils.closeQuietly(inputStm);
                                    mimeType[0] = (String) resultProperties.get(Properties.PROP_RESULT_MIME_TYPE);
                                }
                            }

                            if (null == resultStm) {
                                resultStm = new ByteArrayInputStream(entryBuffer);
                                mimeType[0] = FileHelper.getMimeType(resourceName);
                            }
                        }
                    }

                    zipInputStm.closeEntry();
                }
            } catch (Exception e) {
                //
            } finally {
                IOUtils.closeQuietly(zipInputStm);
                IOUtils.closeQuietly(documentStm);
            }
        }

        // !!! Possible resource leak warning has been checked:
        // returned resultStm is closed by caller of this method
        return resultStm;
    }

    // - Members ---------------------------------------------------------------

    private ResourceManager resourceManager = null;
}
